package earth.krakatoa.core.mongo;

import com.mongodb.ConnectionString;
import com.mongodb.MongoClientSettings;
import com.mongodb.reactivestreams.client.MongoClient;
import com.mongodb.reactivestreams.client.MongoClients;
import com.mongodb.reactivestreams.client.MongoCollection;
import com.mongodb.reactivestreams.client.MongoDatabase;
import earth.krakatoa.core.config.MongoConfig;
import lombok.Data;
import org.bson.Document;

@Data
public class MongoManager {

  private ConnectionString connectionString;

  private MongoClientSettings mongoClientSettings;
  private MongoClient mongoClient;

  private MongoConfig mongoConfig;

  private MongoDatabase krakatoaDatabase;

  private MongoCollection<Document> playersCollection;

  public MongoManager(MongoConfig mongoConfig) {
    this.mongoConfig = mongoConfig;
  }

  public void connect() {
    this.connectionString = new ConnectionString(
        "mongodb://" + mongoConfig.getUsername() + ":" + mongoConfig.getPassword() + "@"
            + mongoConfig.getHostname() + ":" + mongoConfig.getPort() + "/?authSource="
            + mongoConfig.getDatabase()
            + "&readPreference=primary&ssl=false");

    this.mongoClientSettings = MongoClientSettings.builder()
        .applyConnectionString(this.connectionString).build();

    this.mongoClient = MongoClients.create(this.mongoClientSettings);

    this.krakatoaDatabase = this.mongoClient.getDatabase("krakatoa");

    this.playersCollection = this.krakatoaDatabase.getCollection("players");
  }

  public void shutdown() {
    this.mongoClient.close();
  }
}
